/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/exception/OrbipayApiException'), require('../swagger/ApiClient'), require('../model/error/Error'), require('./../util/CommonUtil'), require('./../constant/RestConstants'), require('./../model/base/ResponseCode'),require('./../constant/HttpParamConstants'));
    }
}(this, function (logger, ApiClient, OrbipayError, CommonUtil, RestConstants, ResponseCode, HttpParamConstants) {
        'use strict';

        /**
         * The OrbipayApiException model module.
         * @module exception/OrbipayApiException
         * @version 1.0.0
         */

        /**
         * Constructs a new <code>OrbipayApiException</code>.
         * @alias module:exception/OrbipayApiException
         * @class
         */
        var exports = function (message, client_request) {
            var _this = this;

            Error.call(_this);
            this.stack = (new Error()).stack;

            _this['message'] = '' + message;
            _this['response_codes'] = [];
            _this['errors'] = [];

            if (client_request) {
                _this['masked_client_request'] = CommonUtil.getMaskedJsonString(client_request);
            }
        };

        exports.prototype = Object.create(Error.prototype);

        exports.prototype.constructor = exports;


        /**
         * Constructs a <code>OrbipayApiException</code> from http response, optionally creating a new instance.
         * Copies all relevant properties from <code>response</code> to <code>obj</code> if supplied or a new instance if not.
         * @param httpErrorMessage
         * @param {Object} response The http response object bearing properties of interest.
         * @param client_request
         * @param {module:exception/OrbipayApiException} obj Optional instance to populate.
         * @return {module:exception/OrbipayApiException} The populated <code>OrbipayApiException</code> instance.
         */
        exports.constructFromHttpResponse = function (httpErrorMessage, response, client_request, obj) {
            obj = obj || new exports(httpErrorMessage, client_request);
            if (response) {
                if (response.hasOwnProperty('status')) {
                    obj['http_response_code'] = response['status'];
                }

                if (response.hasOwnProperty('masked_http_request')) {
                    obj['masked_http_request'] = response['masked_http_request'];
                }

                if (response.hasOwnProperty('masked_authorization_data')) {
                    obj['masked_authorization_data'] = response['masked_authorization_data'];
                }


                if (response.hasOwnProperty('text')) {

                    obj['response_body'] = response['text'];

                    if (CommonUtil.IsJsonString(response['text'])) {
                        var errorResponse = CommonUtil.parseJSON(response['text']);
                        if (errorResponse && errorResponse.hasOwnProperty('errors') && Array.isArray(errorResponse['errors'])) {
                            obj['errors'] = errorResponse['errors'].map(function (item) {
                                return OrbipayError.constructFromObject(item);
                            });
                        }
                    }
                }

                if (response.hasOwnProperty('header')) {
                    var http_response_headers = response['header'];
                    obj['http_response_headers'] = http_response_headers;

                    if (http_response_headers.hasOwnProperty('response_codes') && http_response_headers.hasOwnProperty('response_codes')) {

                        var responseCodesHeader = http_response_headers['response_codes'] ? http_response_headers['response_codes'].split(',') : undefined;
                        var responseTextHeader = http_response_headers['response_text'] ? http_response_headers['response_text'].split('|') : undefined;

                        if (responseCodesHeader && responseTextHeader && responseCodesHeader.length === responseTextHeader.length) {
                            var responseCodes = [];
                            for (var i = 0; i < responseCodesHeader.length; i++) {
                                responseCodes.push(new ResponseCode(responseCodesHeader[i], responseTextHeader[i]));
                            }
                            obj['response_codes'] = responseCodes;
                        }
                    }
                }
            }
            return obj;
        };

        exports.prototype.addFieldErrorAndResponseCodes = function (field, appErrorCode) {
            var _this = this;
            if (Array.isArray(_this['response_codes']) && Array.isArray(_this['errors'])) {
                _this['errors'].push(OrbipayError.getFieldError(field));
                _this['response_codes'].push(new ResponseCode(appErrorCode, RestConstants.ERROR_FIELD_MESSAGE));
            }
        };

        exports.getDefaultException = function (error, client_request) {
            var exception = new exports(error.message, client_request);
            exception['stack'] = error.stack;
            exception[HttpParamConstants.ERRORS].push(OrbipayError.getTechDiffError());
            exception['response_codes'].push(new ResponseCode(RestConstants.ERROR_TECH_DIFFICULTIES_APP_ERROR_CODE, RestConstants.ERROR_TECH_DIFFICULTIES_MESSAGE));
            return exception;
        };

        /**
         * Base64 encoded summary used for debugging purpose.
         * @return {String} debugSummary
         */
        exports.prototype.getDebugSummary = function () {
            var debugSummary = '-------------------------START-------------------------\n';
            debugSummary += '\n\nMESSAGE: ' + this['message'] + '\n';
            if (this.hasOwnProperty('masked_client_request')) {
                debugSummary += '\n\nMASKED_CLIENT_REQUEST: ' + this['masked_client_request'] + '\n';
            }
            if (this.hasOwnProperty('masked_http_request')) {
                debugSummary += '\n\nMASKED_HTTP_REQUEST: ' + this['masked_http_request'] + '\n';
            }
            if (this.hasOwnProperty('masked_authorization_data')) {
                debugSummary += '\n\nMASKED_AUTHORIZATION_DATA: ' + this['masked_authorization_data'] + '\n';
            }
            if (this.hasOwnProperty('stack')) {
                debugSummary += '\n\nSTACK: ' + this['stack'] + '\n';
            }
            if (this.hasOwnProperty('response_body')) {
                debugSummary += '\n\nRESPONSE_BODY: ' + this['response_body'] + '\n';
            }
            if (this.hasOwnProperty('http_response_code')) {
                debugSummary += '\n\nHTTP_RESPONSE_CODE: ' + this['http_response_code'] + '\n';
            }
            if (this.hasOwnProperty('http_response_headers')) {
                debugSummary += '\n\nHTTP_RESPONSE_HEADERS: ' + JSON.stringify(this['http_response_headers']) + '\n';
            }
            if (this.hasOwnProperty('errors')) {
                debugSummary += '\n\nERRORS: ' + JSON.stringify(this['errors']) + '\n';
            }
            if (this.hasOwnProperty('response_codes')) {
                debugSummary += '\n\nRESPONSE_CODES: ' + JSON.stringify(this['response_codes']) + '\n';
            }
            debugSummary += '-------------------------END-------------------------';
            return Buffer.from(debugSummary).toString('base64');
        };


        /**
         * @member {String} name
         */
        exports.prototype['name'] = "OrbipayApiException";
        /**
         * @member {String} message
         */
        exports.prototype['message'] = undefined;
        /**
         * @member {String} http_response_code
         */
        exports.prototype['http_response_code'] = undefined;
        /**
         * @member {Array.<module:model/error/Error>} errors
         */
        exports.prototype['errors'] = undefined;
        /**
         * @member {Array.<String>} http_response_headers
         */
        exports.prototype['http_response_headers'] = undefined;
        /**
         * @member {Array.<module:model/base/ResponseCode>} response_codes
         */
        exports.prototype['response_codes'] = undefined;
        /**
         * @member {String} masked_client_request
         */
        exports.prototype['masked_client_request'] = undefined;
        /**
         * @member {String} masked_http_request
         */
        exports.prototype['masked_http_request'] = undefined;
        /**
         * @member {String} masked_authorization_data
         */
        exports.prototype['masked_authorization_data'] = undefined;
        /**
         * @member {String} response_body
         */
        exports.prototype['response_body'] = undefined;

        return exports;
    }
))
;
