/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/customer/RetrieveCustomersHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/CustomerApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, CustomerApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The RetrieveCustomersHandler model module.
     * @module handler/customer/RetrieveCustomersHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>RetrieveCustomersHandler</code>.
     * @alias module:handler/customer/RetrieveCustomersHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (customers, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(customers);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(customers, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, customers);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (customers) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, customers);

        ValidationUtil.validate(customers[HttpParamConstants.PAGE_SIZE], DataTypeEnum.String, MandatoryEnum.false, exception, HttpParamConstants.PAGE_SIZE, AppErrorCodes.PAGE_SIZE_INVALID, ValidationUtil.isValidPageSize);
        ValidationUtil.validate(customers[HttpParamConstants.CUSTOMER_REFERENCE], DataTypeEnum.String, MandatoryEnum.false, exception, HttpParamConstants.CUSTOMER_REFERENCE, AppErrorCodes.CUSTOMER_REFERENCE_INVALID, ValidationUtil.isValidCustomerReference);
        ValidationUtil.validate(customers[HttpParamConstants.SSN], DataTypeEnum.String, MandatoryEnum.false, exception, HttpParamConstants.SSN, AppErrorCodes.SSN_INVALID, ValidationUtil.isValidSSN);
        ValidationUtil.validate(customers[HttpParamConstants.EMAIL], DataTypeEnum.String, MandatoryEnum.false, exception, HttpParamConstants.EMAIL, AppErrorCodes.EMAIL_INVALID, ValidationUtil.isValidEmail);
        ValidationUtil.validate(customers[HttpParamConstants.ACCOUNT_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception, HttpParamConstants.ACCOUNT_NUMBER, AppErrorCodes.CUSTOMER_ACCOUNT_NUMBER_INVALID, ValidationUtil.isValidCustomerAccountNumber);
        ValidationUtil.validateInvocationContext(customers[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(customers[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (customers) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(customers[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'pageSize': customers[HttpParamConstants.PAGE_SIZE],
            'customerReference': customers[HttpParamConstants.CUSTOMER_REFERENCE],
            'ssn': customers[HttpParamConstants.SSN],
            'email': customers[HttpParamConstants.EMAIL],
            'accountNumber': customers[HttpParamConstants.ACCOUNT_NUMBER]
        };
    };

    exports.prototype.callAPI = function (customers, callback) {
        var _this = this;
        var invocationContext = customers[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var opts = _this.getOptions(customers);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], customers[HttpParamConstants.API_END_POINT]);
        var customerApi = new CustomerApi(apiClient);
        var request = customerApi.retrieveCustomers(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


