/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/customer/UpdateCustomerHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/CustomerApi'), require('./../../util/CommonUtil'), require('../../swagger/model/UpdateCustomerRequest'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, CustomerApi, CommonUtil, UpdateCustomerRequest, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The UpdateCustomerHandler model module.
     * @module handler/customer/UpdateCustomerHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>UpdateCustomerHandler</code>.
     * @alias module:handler/customer/UpdateCustomerHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (customer, callback) {
        var response = {};
        var exception;
        try {
            exception = exports.validate(customer);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(customer, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, customer);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.validate = function (customer) {

        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, customer);

        ValidationUtil.validate(customer[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        exports.validateCustomer(customer, exception);

        ValidationUtil.validateInvocationContext(customer[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(customer[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.validateCustomerAccount = function (customerAccount, exception, segment) {
        segment += HttpParamConstants.DOT;
        ValidationUtil.validate(customerAccount[HttpParamConstants.ACCOUNT_HOLDER_NAME], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.ACCOUNT_HOLDER_NAME, AppErrorCodes.CUSTOMER_ACCOUNT_HOLDER_NAME_INVALID, ValidationUtil.isValidAccountHolderName);

        ValidationUtil.validate(customerAccount[HttpParamConstants.NICKNAME], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.NICKNAME, AppErrorCodes.CUSTOMER_ACCOUNT_NICKNAME_INVALID, ValidationUtil.isValidNickName);

        ValidationUtil.validate(customerAccount[HttpParamConstants.CUSTOMER_ACCOUNT_REFERENCE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.CUSTOMER_ACCOUNT_REFERENCE, AppErrorCodes.CUSTOMER_ACCOUNT_REFERENCE_INVALID, ValidationUtil.isValidCustomerAccountReference);

        ValidationUtil.validate(customerAccount[HttpParamConstants.ACCOUNT_NUMBER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            segment + HttpParamConstants.ACCOUNT_NUMBER, AppErrorCodes.CUSTOMER_ACCOUNT_NUMBER_INVALID, ValidationUtil.isValidCustomerAccountNumber);

        ValidationUtil.validate(customerAccount[HttpParamConstants.CURRENT_BALANCE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.ACCOUNT_NUMBER, AppErrorCodes.CURRENT_BALANCE_INVALID, ValidationUtil.isValidCustomerAccountBalance);

        ValidationUtil.validate(customerAccount[HttpParamConstants.CURRENT_STATEMENT_BALANCE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.CURRENT_STATEMENT_BALANCE, AppErrorCodes.CURRENT_STATEMENT_BALANCE_INVALID, ValidationUtil.isValidCustomerAccountBalance);

        ValidationUtil.validate(customerAccount[HttpParamConstants.MINIMUM_PAYMENT_DUE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.MINIMUM_PAYMENT_DUE, AppErrorCodes.MINIMUM_PAYMENT_DUE_INVALID, ValidationUtil.isValidCustomerAccountBalance);

        ValidationUtil.validate(customerAccount[HttpParamConstants.PAST_AMOUNT_DUE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.PAST_AMOUNT_DUE, AppErrorCodes.PAST_AMOUNT_DUE_INVALID, ValidationUtil.isValidCustomerAccountBalance);

        ValidationUtil.validate(customerAccount[HttpParamConstants.PAYMENT_DUE_DATE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.PAYMENT_DUE_DATE, AppErrorCodes.PAYMENT_DUE_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(customerAccount[HttpParamConstants.STATEMENT_DATE], DataTypeEnum.String, MandatoryEnum.no, exception,
            segment + HttpParamConstants.STATEMENT_DATE, AppErrorCodes.STATEMENT_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(customerAccount[HttpParamConstants.CUSTOM_FIELDS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            segment + HttpParamConstants.CUSTOM_FIELDS, AppErrorCodes.CUSTOM_FIELDS_INVALID, ValidationUtil.validateCustomFields);

        ValidationUtil.validate(customerAccount[HttpParamConstants.ADDRESS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            segment + HttpParamConstants.ADDRESS, AppErrorCodes.SYSTEM_ERROR, ValidationUtil.validateCustomerAccountAddress);
    };

    exports.validateCustomerAccounts = function (customerAccounts, exception) {
        ValidationUtil.validate(customerAccounts[HttpParamConstants.LIST], DataTypeEnum.Array, MandatoryEnum.no, exception,
            HttpParamConstants.CUSTOMER_ACCOUNTS, AppErrorCodes.ERROR_SEGMENT_MISSING, exports.validateCustomerAccountsList);
    };

    exports.validateCustomerAccountsList = function (list, exception, segment) {
        for (var i = 0; i < list.length; i++) {
            var baseSegment = segment + HttpParamConstants.LEFT_SQUARE_BRACE + i + HttpParamConstants.RIGHT_SQUARE_BRACE;
            ValidationUtil.validate(list[i], DataTypeEnum.Object, MandatoryEnum.yes, exception,
                baseSegment, AppErrorCodes.ERROR_SEGMENT_MISSING, exports.validateCustomerAccount);
        }
    };

    exports.validateCustomer = function (customer, exception) {

        ValidationUtil.validate(customer[HttpParamConstants.COMMENTS], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.COMMENTS, AppErrorCodes.COMMENTS_INVALID, ValidationUtil.isValidComments);

        ValidationUtil.validate(customer[HttpParamConstants.CUSTOMER_REFERENCE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.CUSTOMER_REFERENCE, AppErrorCodes.CUSTOMER_REFERENCE_INVALID, ValidationUtil.isValidCustomerReference);

        ValidationUtil.validate(customer[HttpParamConstants.FIRST_NAME], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.FIRST_NAME, AppErrorCodes.FIRST_NAME_INVALID, ValidationUtil.isValidName);

        ValidationUtil.validate(customer[HttpParamConstants.LAST_NAME], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.LAST_NAME, AppErrorCodes.LAST_NAME_INVALID, ValidationUtil.isValidName);

        ValidationUtil.validate(customer[HttpParamConstants.MIDDLE_NAME], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.MIDDLE_NAME, AppErrorCodes.MIDDLE_NAME_INVALID, ValidationUtil.isValidName);

        ValidationUtil.validate(customer[HttpParamConstants.GENDER], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.GENDER, AppErrorCodes.GENDER_INVALID, ValidationUtil.isValidGender);

        ValidationUtil.validate(customer[HttpParamConstants.DATE_OF_BIRTH], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.DATE_OF_BIRTH, AppErrorCodes.DATE_OF_BIRTH_INVALID, ValidationUtil.isValidDateOfBirth);

        ValidationUtil.validate(customer[HttpParamConstants.SSN], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.SSN, AppErrorCodes.SSN_INVALID, ValidationUtil.isValidSSN);

        ValidationUtil.validate(customer[HttpParamConstants.LOCALE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.LOCALE, AppErrorCodes.LOCALE_INVALID, ValidationUtil.isValidLocale);

        ValidationUtil.validate(customer[HttpParamConstants.EMAIL], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.EMAIL, AppErrorCodes.EMAIL_INVALID, ValidationUtil.isValidEmail);

        ValidationUtil.validate(customer[HttpParamConstants.HOME_PHONE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.HOME_PHONE, AppErrorCodes.HOME_PHONE_INVALID, ValidationUtil.isValidPhone);

        ValidationUtil.validate(customer[HttpParamConstants.WORK_PHONE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.WORK_PHONE, AppErrorCodes.WORK_PHONE_INVALID, ValidationUtil.isValidPhone);

        ValidationUtil.validate(customer[HttpParamConstants.MOBILE_PHONE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.MOBILE_PHONE, AppErrorCodes.MOBILE_PHONE_INVALID, ValidationUtil.isValidPhone);

        ValidationUtil.validate(customer[HttpParamConstants.CUSTOM_FIELDS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.CUSTOM_FIELDS, AppErrorCodes.CUSTOM_FIELDS_INVALID, ValidationUtil.validateCustomFields);

        ValidationUtil.validate(customer[HttpParamConstants.ADDRESS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.ADDRESS, AppErrorCodes.ERROR_SEGMENT_MISSING, ValidationUtil.validateCustomerAddress);

        ValidationUtil.validate(customer[HttpParamConstants.CUSTOMER_ACCOUNTS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.CUSTOMER_ACCOUNTS, AppErrorCodes.ERROR_SEGMENT_MISSING, exports.validateCustomerAccounts);
    };

    exports.prototype.getOptions = function (customer) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(customer[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (customer, callback) {
        var invocationContext = customer[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        if (customer['customer_accounts']) {
            customer['customer_accounts'] = customer['customer_accounts']['list'];
        }

        var updateCustomerRequest = UpdateCustomerRequest.constructFromObject(customer);

        var opts = this.getOptions(customer);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], customer[HttpParamConstants.API_END_POINT]);
        var customerApi = new CustomerApi(apiClient);
        var request = customerApi.updateCustomer(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType, customer[HttpParamConstants.PATH_PARAM_ID_CUSTOMER],
            updateCustomerRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));
