/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/customer/RetrieveCustomerAccountsHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/CustomerAccountApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, CustomerAccountApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The RetrieveCustomerAccountsHandler model module.
     * @module handler/customer/RetrieveCustomerAccountsHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>RetrieveCustomerAccountsHandler</code>.
     * @alias module:handler/customer/RetrieveCustomerAccountsHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (customerAccounts, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(customerAccounts);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(customerAccounts, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, customerAccounts);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (customerAccounts) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, customerAccounts);

        ValidationUtil.validate(customerAccounts[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.true, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(customerAccounts[HttpParamConstants.PAGE_SIZE], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.PAGE_SIZE, AppErrorCodes.PAGE_SIZE_INVALID, ValidationUtil.isValidPageSize);
        ValidationUtil.validate(customerAccounts[HttpParamConstants.ACCOUNT_NUMBER], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.ACCOUNT_NUMBER, AppErrorCodes.CUSTOMER_ACCOUNT_NUMBER_INVALID, ValidationUtil.isValidCustomerAccountNumber);
        ValidationUtil.validate(customerAccounts[HttpParamConstants.STATUS], DataTypeEnum.Array, MandatoryEnum.false, exception,
            HttpParamConstants.STATUS, AppErrorCodes.CUSTOMER_ACCOUNT_STATUS_INVALID, ValidationUtil.isValidCustomerAccountStatuses);

        ValidationUtil.validate(customerAccounts[HttpParamConstants.QUERY_ID], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.QUERY_ID, AppErrorCodes.QUERY_ID_INVALID, ValidationUtil.isValidQueryId);
        ValidationUtil.validate(customerAccounts[HttpParamConstants.FROM_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.FROM_INDEX, AppErrorCodes.FROM_INDEX_INVALID, ValidationUtil.isValidFromIndex);
        ValidationUtil.validate(customerAccounts[HttpParamConstants.TO_INDEX], DataTypeEnum.String, MandatoryEnum.false, exception,
            HttpParamConstants.TO_INDEX, AppErrorCodes.TO_INDEX_INVALID, ValidationUtil.isValidToIndex);

        ValidationUtil.validateInvocationContext(customerAccounts[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(customerAccounts[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (customerAccounts) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(customerAccounts[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'pageSize': customerAccounts[HttpParamConstants.PAGE_SIZE],
            'status': customerAccounts[HttpParamConstants.STATUS],
            'accountNumber': customerAccounts[HttpParamConstants.ACCOUNT_NUMBER]
        };
    };

    exports.prototype.getPageOptions = function (customerAccounts) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(customerAccounts[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES]),
            'pageSize': customerAccounts[HttpParamConstants.PAGE_SIZE],
            'fromIndex': customerAccounts[HttpParamConstants.FROM_INDEX],
            'toIndex': customerAccounts[HttpParamConstants.TO_INDEX]
        };
    };

    exports.prototype.callAPI = function (customerAccounts, callback) {
        var _this = this;
        var invocationContext = customerAccounts[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];
        var queryId = customerAccounts[HttpParamConstants.QUERY_ID];
        var ID_CUSTOMER = customerAccounts[HttpParamConstants.PATH_PARAM_ID_CUSTOMER];

        var opts;
        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], customerAccounts[HttpParamConstants.API_END_POINT]);
        var customerAccountApi = new CustomerAccountApi(apiClient);
        var request;
        if (ValidationUtil.isNonEmptyString(customerAccounts[HttpParamConstants.QUERY_ID])) {
            opts = _this.getPageOptions(customerAccounts);
            request = customerAccountApi.retrieveCustomerAccountsPage(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType, ID_CUSTOMER, queryId, opts, callback);
        } else {
            opts = _this.getOptions(customerAccounts);
            request = customerAccountApi.retrieveCustomerAccounts(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType, ID_CUSTOMER, opts, callback);
        }

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));
