/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/payment/CreatePaymentHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/PaymentApi'), require('./../../util/CommonUtil'), require('../../swagger/model/CreatePaymentRequest'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, PaymentApi, CommonUtil, CreatePaymentRequest, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The CreatePaymentHandler model module.
     * @module handler/payment/CreatePaymentHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>CreatePaymentHandler</code>.
     * @alias module:handler/payment/CreatePaymentHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (payment, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(payment);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(payment, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, payment);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (payment) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, payment);

        ValidationUtil.validate(payment[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        this.validatePayment(payment, exception);
        ValidationUtil.validateInvocationContext(payment[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(payment[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };


    exports.prototype.validatePayment = function (payment, exception) {

        ValidationUtil.validate(payment[HttpParamConstants.AMOUNT], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.AMOUNT, AppErrorCodes.AMOUNT_INVALID, ValidationUtil.isValidAmount);

        ValidationUtil.validate(payment[HttpParamConstants.CARD_CVV_NUMBER], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.CARD_CVV_NUMBER, AppErrorCodes.CARD_CVV_NUMBER_INVALID, ValidationUtil.isValidCardCVVNumber);

        ValidationUtil.validate(payment[HttpParamConstants.PAYMENT_DATE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.PAYMENT_DATE, AppErrorCodes.PAYMENT_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(payment[HttpParamConstants.PAYMENT_REQUEST_DATE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.PAYMENT_REQUEST_DATE, AppErrorCodes.PAYMENT_REQUEST_DATE_INVALID, ValidationUtil.isValidDate);

        ValidationUtil.validate(payment[HttpParamConstants.PAYMENT_AMOUNT_TYPE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.PAYMENT_AMOUNT_TYPE, AppErrorCodes.PAYMENT_AMOUNT_TYPE_INVALID, ValidationUtil.isValidPaymentAmountType);

        ValidationUtil.validate(payment[HttpParamConstants.PAYMENT_REFERENCE], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.PAYMENT_REFERENCE, AppErrorCodes.PAYMENT_REFERENCE_INVALID, ValidationUtil.isValidPaymentReference);

        ValidationUtil.validate(payment[HttpParamConstants.COMMENTS], DataTypeEnum.String, MandatoryEnum.no, exception,
            HttpParamConstants.COMMENTS, AppErrorCodes.COMMENTS_INVALID, ValidationUtil.isValidComments);

        ValidationUtil.validate(payment[HttpParamConstants.CUSTOM_FIELDS], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.CUSTOM_FIELDS, AppErrorCodes.CUSTOM_FIELDS_INVALID, ValidationUtil.validateCustomFields);

        ValidationUtil.validate(payment[HttpParamConstants.FEE], DataTypeEnum.Object, MandatoryEnum.no, exception,
            HttpParamConstants.FEE, null, this.validateFee);

        ValidationUtil.validate(payment[HttpParamConstants.CUSTOMER], DataTypeEnum.Object, MandatoryEnum.yes, exception,
            HttpParamConstants.CUSTOMER, AppErrorCodes.ERROR_SEGMENT_MISSING, this.validateCustomer);

        ValidationUtil.validate(payment[HttpParamConstants.CUSTOMER_ACCOUNT], DataTypeEnum.Object, MandatoryEnum.yes, exception,
            HttpParamConstants.CUSTOMER_ACCOUNT, AppErrorCodes.ERROR_SEGMENT_MISSING, this.validateCustomerAccount);

        ValidationUtil.validate(payment[HttpParamConstants.FUNDING_ACCOUNT], DataTypeEnum.Object, MandatoryEnum.yes, exception,
            HttpParamConstants.FUNDING_ACCOUNT, AppErrorCodes.ERROR_SEGMENT_MISSING, this.validateFundingAccount);

    };

    exports.prototype.validateCustomerAccount = function (customerAccount, exception, segment) {
        segment += HttpParamConstants.DOT;
        ValidationUtil.validate(customerAccount[HttpParamConstants.ID], DataTypeEnum.String, MandatoryEnum.yes,
            exception, segment + HttpParamConstants.ID, AppErrorCodes.ID_CUSTOMER_ACCOUNT_INVALID, ValidationUtil.isValidId);
    };

    exports.prototype.validateFundingAccount = function (fundingAccount, exception, segment) {
        segment += HttpParamConstants.DOT;
        ValidationUtil.validate(fundingAccount[HttpParamConstants.ID], DataTypeEnum.String, MandatoryEnum.yes,
            exception, segment + HttpParamConstants.ID, AppErrorCodes.ID_FUNDING_ACCOUNT_INVALID, ValidationUtil.isValidId);
    };

    exports.prototype.validateFee = function (fee, exception, segment) {
        segment += HttpParamConstants.DOT;
        ValidationUtil.validate(fee[HttpParamConstants.FEE_AMOUNT], DataTypeEnum.String, MandatoryEnum.no,
            exception, segment + HttpParamConstants.FEE_AMOUNT, AppErrorCodes.FEE_AMOUNT_INVALID, ValidationUtil.isValidAmount);
    };

    exports.prototype.validateCustomer = function (customer, exception, segment) {
        segment += HttpParamConstants.DOT;
        ValidationUtil.validate(customer[HttpParamConstants.ID], DataTypeEnum.String, MandatoryEnum.yes,
            exception, segment + HttpParamConstants.ID, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);
    };

    exports.prototype.getOptions = function (payment) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(payment[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (payment, callback) {
        var invocationContext = payment[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var createPaymentRequest = CreatePaymentRequest.constructFromObject(payment);

        var opts = this.getOptions(payment);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], payment[HttpParamConstants.API_END_POINT]);
        var paymentApi = new PaymentApi(apiClient);
        var request = paymentApi.createPayment(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            payment[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], createPaymentRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


