/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/customer/Customer'), require('./../base/Address'), require('./../customeraccount/CustomerAccounts'), require('../fundingaccount/FundingAccounts'), require('./../base/Base'), require('./../base/InvocationContext'), require('./../../enum/ProductEnum'), require('./../base/Attributes'), require('./../../constant/RestConstants'), require('./../../enum/InvocationModeEnum'), require('../../handler/customer/CreateCustomerHandler'), require('../../exception/OrbipayApiException'), require('./../../util/CommonUtil'), require('./../../util/ValidationUtil'), require('./../../handler/customer/GetCustomerHandler'), require('./../../handler/customer/UpdateCustomerHandler'), require('./../../util/ResponseWrapper'));
    }
}(this, function (logger, Address, CustomerAccounts, FundingAccounts, Base, InvocationContext, ProductEnum, Attributes, RestConstants, InvocationModeEnum, CreateCustomerHandler, OrbipayApiException, CommonUtil, ValidationUtil, GetCustomerHandler, UpdateCustomerHandler, ResponseWrapper) {
    'use strict';

    /**
     * The Customer model module.
     * @module model/customer/Customer
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Customer</code>.
     * @alias module:model/customer/Customer
     * @class
     * @param ID_CUSTOMER {String} The unique identifier assigned by Orbipay to the customer.
     */
    var exports = function (ID_CUSTOMER) {
        var _this = this;
        Base.call(_this);
        _this['ID_CUSTOMER'] = ID_CUSTOMER;
        _this['id'] = ID_CUSTOMER;
    };

    /**
     * @private
     * Constructs a <code>Customer</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/customer/Customer} obj Optional instance to populate.
     * @return {module:model/customer/Customer} The populated <code>Customer</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('first_name')) {
                obj['first_name'] = data['first_name'];
            }
            if (data.hasOwnProperty('last_name')) {
                obj['last_name'] = data['last_name'];
            }
            if (data.hasOwnProperty('middle_name')) {
                obj['middle_name'] = data['middle_name'];
            }
            if (data.hasOwnProperty('gender')) {
                obj['gender'] = data['gender'];
            }
            if (data.hasOwnProperty('date_of_birth')) {
                obj['date_of_birth'] = data['date_of_birth'];
            }
            if (data.hasOwnProperty('ssn')) {
                obj['ssn'] = data['ssn'];
            }
            if (data.hasOwnProperty('locale')) {
                obj['locale'] = data['locale'];
            }
            if (data.hasOwnProperty('email')) {
                obj['email'] = data['email'];
            }
            if (data.hasOwnProperty('home_phone')) {
                obj['home_phone'] = data['home_phone'];
            }
            if (data.hasOwnProperty('work_phone')) {
                obj['work_phone'] = data['work_phone'];
            }
            if (data.hasOwnProperty('mobile_phone')) {
                obj['mobile_phone'] = data['mobile_phone'];
            }
            if (data.hasOwnProperty('address')) {
                obj['address'] = Address.constructFromObject(data['address']);
            }
            if (data.hasOwnProperty('customer_reference')) {
                obj['customer_reference'] = data['customer_reference'];
            }
            if (data.hasOwnProperty('id')) {
                obj['id'] = data['id'];
            }
            if (data.hasOwnProperty('status')) {
                obj['status'] = data['status'];
            }
            if (data.hasOwnProperty('url')) {
                obj['url'] = data['url'];
            }
            if (data.hasOwnProperty('customer_accounts')) {
                obj['customer_accounts'] = CustomerAccounts.constructFromObject(data['customer_accounts']);
            }
            if (data.hasOwnProperty('funding_accounts')) {
                obj['funding_accounts'] = FundingAccounts.constructFromObject(data['funding_accounts']);
            }
            if (data.hasOwnProperty('comments')) {
                obj['comments'] = data['comments'];
            }

            //Lazy loading to avoid circular dependency.
            if (typeof module === 'object' && module.exports) {
                if (data.hasOwnProperty('payments')) {
                    var Payments = require('./../payment/Payments');
                    obj['payments'] = Payments.constructFromObject(data['payments']);
                }
                if (data.hasOwnProperty('payment_setups')) {
                    var PaymentSetups = require('./../paymentsetup/PaymentSetups');
                    obj['payment_setups'] = PaymentSetups.constructFromObject(data['payment_setups']);
                }
            }

            if (data.hasOwnProperty('custom_fields')) {
                obj['custom_fields'] = data['custom_fields'];
            }
        }
        return obj;
    };


    exports.prototype = Object.create(Base.prototype);

    exports.prototype.constructor = exports;

    /**
     * The first name of the customer.
     * @member {String} first_name
     */
    exports.prototype['first_name'] = undefined;
    /**
     * The last name of the customer.
     * @member {String} last_name
     */
    exports.prototype['last_name'] = undefined;
    /**
     * The middle name of the customer.
     * @member {String} middle_name
     */
    exports.prototype['middle_name'] = undefined;
    /**
     * The gender of the customer.
     * @member {module:model/customer/Customer.GenderEnum} gender
     */
    exports.prototype['gender'] = undefined;
    /**
     * The date of birth of the customer. It should be in the full-date format as per ISO8601, namely, YYYY-MM-DD.
     * @member {String} date_of_birth
     */
    exports.prototype['date_of_birth'] = undefined;
    /**
     * The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.
     * @member {String} ssn
     */
    exports.prototype['ssn'] = undefined;
    /**
     * The language in which the payee wants the alerts and notifications from Orbipay. This is to be specified in the format,<br><i>< <b>ISO-639-1 language code</b>  >_< <b>ISO ALPHA-2 Country Code</b> ></i>.<br> For e.g., <b>en_US</b> indicates the language preference as US English, which is also the default value.
     * @member {String} locale
     */
    exports.prototype['locale'] = undefined;
    /**
     * The email address of the customer.
     * @member {String} email
     */
    exports.prototype['email'] = undefined;
    /**
     * The home phone number of the customer.
     * @member {String} home_phone
     */
    exports.prototype['home_phone'] = undefined;
    /**
     * The work phone number of the customer.
     * @member {String} work_phone
     */
    exports.prototype['work_phone'] = undefined;
    /**
     * The mobile phone number of the customer.
     * @member {String} mobile_phone
     */
    exports.prototype['mobile_phone'] = undefined;
    /**
     * @member {module:model/base/Address} address
     */
    exports.prototype['address'] = undefined;
    /**
     * The unique identifier in the client system for the customer.
     * @member {String} customer_reference
     */
    exports.prototype['customer_reference'] = undefined;
    /**
     * The unique identifier assigned by Orbipay to the customer.
     * @member {String} id
     */
    exports.prototype['id'] = undefined;
    /**
     * The status of the customer in Orbipay.
     * @member {module:model/customer/Customer.StatusEnum} status
     */
    exports.prototype['status'] = undefined;
    /**
     * This URL fetches the Customer details.
     * @member {String} url
     */
    exports.prototype['url'] = undefined;
    /**
     * @member {module:model/customeraccount/CustomerAccounts} customer_accounts
     */
    exports.prototype['customer_accounts'] = undefined;
    /**
     * @member {module:model/fundingaccount/FundingAccounts} funding_accounts
     */
    exports.prototype['funding_accounts'] = undefined;
    /**
     * @member {module:model/payment/Payments} payments
     */
    exports.prototype['payments'] = undefined;
    /**
     * @member {module:model/paymentsetup/PaymentSetups} payment_setups
     */
    exports.prototype['payment_setups'] = undefined;
    /**
     * The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact support@billerpayments.com for more information on configuring and using custom fields.
     * @member {Object} custom_fields
     */
    exports.prototype['custom_fields'] = undefined;
    /**
     * This URL fetches the Customer details.
     * @member {String} comments
     */
    exports.prototype['comments'] = undefined;

    /**
     * @param {module:model/base/Address} address.
     * @return module:model/customer/Customer
     */
    exports.prototype.withAddress = function (address) {
        var _this = this;
        _this['address'] = Address.constructFromObject(address);
        return _this;
    };

    /**
     * @param {String} comments.
     * @return module:model/customer/Customer
     */
    exports.prototype.withMemo = function (comments) {
        var _this = this;
        _this['comments'] = comments;
        return _this;
    };

    /**
     * @param {module:model/customeraccount/CustomerAccounts} customer_accounts.
     * @return module:model/customer/Customer
     */
    exports.prototype.withAccounts = function (customer_accounts) {
        var _this = this;
        _this['customer_accounts'] = CustomerAccounts.constructFromObject(customer_accounts);
        return _this;
    };

    /**
     * @param {String} first_name
     * @param {String} last_name
     * @param {String} middle_name
     * @param {String} date_of_birth
     * @param {String} ssn
     * @param {String} gender
     * @return module:model/customer/Customer
     */
    exports.prototype.withDetails = function (first_name, last_name, middle_name, date_of_birth, ssn, gender) {
        var _this = this;
        _this['first_name'] = first_name;
        _this['last_name'] = last_name;
        _this['middle_name'] = middle_name;
        _this['date_of_birth'] = date_of_birth;
        _this['ssn'] = ssn;
        _this['gender'] = gender;
        return _this;
    };

    /**
     * @param {String} customer_reference
     * @return module:model/customer/Customer
     */
    exports.prototype.withReference = function (customer_reference) {
        var _this = this;
        _this['customer_reference'] = customer_reference;
        return _this;
    };

    /**
     * @param {String} status. Use {module:enum/CustomerStatusEnum} to get values.
     * @return module:model/customer/Customer
     */
    exports.prototype.withStatus = function (status) {
        var _this = this;
        _this['status'] = status;
        return _this;
    };

    /**
     * @param {String} email
     * @param {String} home_phone
     * @param {String} work_phone
     * @param {String} mobile_phone
     * @return module:model/customer/Customer
     */
    exports.prototype.withContactInfo = function (email, home_phone, work_phone, mobile_phone) {
        var _this = this;
        _this['email'] = email;
        _this['home_phone'] = home_phone;
        _this['work_phone'] = work_phone;
        _this['mobile_phone'] = mobile_phone;
        return _this;
    };

    /**
     * @param {String} language
     * @param {String} country
     * @return module:model/customer/Customer
     */
    exports.prototype.withLocale = function (language, country) {
        var _this = this;
        if (ValidationUtil.isNonEmptyString(language) && ValidationUtil.isNonEmptyString(country)) {
            _this['locale'] = language + RestConstants.LOCALE_DELIMITER + country;
        }
        return _this;
    };

    /**
     * @param {Object} custom_fields
     * @return module:model/customer/Customer
     */
    exports.prototype.withCustomFields = function (custom_fields) {
        var _this = this;
        _this['custom_fields'] = custom_fields;
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/customer/Customer
     */
    exports.prototype.createdBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/customer/Customer
     */
    exports.prototype.retrievedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/customer/Customer
     */
    exports.prototype.updatedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} channel
     * @return module:model/customer/Customer
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        Base.prototype.videChannel.call(_this, channel);
        return _this;
    };

    /**
     * @param {String} client_key
     * @return module:model/customer/Customer
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        Base.prototype.forClient.call(_this, client_key);
        return _this;
    };

    /**
     * Callback function to receive the result of the operation.
     * @callback module:model/customer/Customer~callApiCallback
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     * @param {module:model/customer/Customer} data The data returned by the service call.
     */

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/customer/Customer~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.create = function (invocation_context, callback, invocation_mode, api_end_point) {
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };
        _this.withContext(invocation_context, invocation_mode, api_end_point);
        var handler = new CreateCustomerHandler();
        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/customer/Customer~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        var handler = new GetCustomerHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/customer/Customer~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.update = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;
        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new UpdateCustomerHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    return exports;
}));


